package jp.co.isid.elportal.fileconvert.command;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import jp.co.isid.elportal.base.cifs.exception.CifsApplicationException;
import jp.co.isid.elportal.base.config.ParameterCatalog;
import jp.co.isid.elportal.base.log.Logger;
import jp.co.isid.elportal.fileconvert.command.process.ProcessExecuter;
import jp.co.isid.elportal.fileconvert.consts.CommonConstants.CifsType;
import jp.co.isid.elportal.fileconvert.util.FileUtil;

/**
 * TiffファイルをJpegファイルに変換するコマンドクラスです。
 *
 *
 */
public class TiffToJpegCommand implements Command<File, String> {

    /** ImageMagicパス **/
    private static final String IMAGE_MAGIC_PATH = ParameterCatalog.instance().getParameter("FileConvertParam", "imageMagicPath");
    /** ImageMagicパラメータ名： quality**/
    private static final String QUALITY = ParameterCatalog.instance().getParameter("FileConvertParam", "quality");
    /** ImageMagicパラメータ値： quality**/
    private static final String JPEG_QUALITY = ParameterCatalog.instance().getParameter("FileConvertParam", "jpegQuality");
    /** ImageMagicパラメータ名： geometry**/
    private static final String GEOMETRY = ParameterCatalog.instance().getParameter("FileConvertParam", "geometry");
    /** ImageMagicパラメータ値： geometry**/
    private static final String JPEG_GEOMETRY = ParameterCatalog.instance().getParameter("FileConvertParam", "jpegGeometry");

    /**
     * デフォルトコンストラクタ
     */
    public TiffToJpegCommand() {
    }

    /**
     * TiffファイルをJpegに変換します。<br>
     *
     * @param tiffFile 変換元ファイル
     * @return 変換後のファイル
     * @throws CommandException 変換に失敗した際にthrowされる例外
     */
    public String execute(File tiffFile) throws CommandException {
        Logger.debug("TiffToJpegCommand execute(File tiffFile)start ----------------------");

        String inFile = tiffFile.getAbsolutePath();

        Logger.debug("tiffFile名::" + inFile + (tiffFile.exists() ? " 存在する" : " 存在しない"));

        if (!tiffFile.exists()) {
            Logger.error("TIFFのJPEG変換に失敗しました。-> TIFFファイルがありません：" + tiffFile.getPath());
            throw new CommandException("TIFFのJPEG変換に失敗しました。");
        }

        // 作成する画像の相対パス取得
        String absolutePath = FileUtil.getAbsolutePath(tiffFile.getName(), "jpg");
        String outFile = null;
        try {
            // 作成する画像の絶対パス取得
            outFile = FileUtil.getFullPath(absolutePath, CifsType.THUMB);
        } catch (CifsApplicationException e) {
            Logger.error("TIFFのJPEG変換に失敗しました。-> JPEG画像ファイルパス取得に失敗しました。", e);
            throw new CommandException("TIFFのJPEG変換に失敗しました。", e);
        }

        Logger.debug("出力JPEGファイルの絶対パス::" + outFile);

        List<String> list = new ArrayList<String>();
        list.add(IMAGE_MAGIC_PATH);
        list.add(QUALITY);
        list.add(JPEG_QUALITY);
        list.add(GEOMETRY);
        list.add(JPEG_GEOMETRY);

        list.add(inFile);
        list.add(outFile);

        ProcessExecuter exec = new ProcessExecuter(list);
        try {
            int returnCd = exec.execute();
            Logger.debug("出力結果::" + returnCd);
            if (returnCd != 0) {
                Logger.error("TIFFのJPEG変換に失敗しました。ERRORCD:" + returnCd);
                throw new CommandException("TIFFのJPEG変換に失敗しました。ERRORCD:" + returnCd);
            }
        } catch (IOException e1) {
            Logger.error("TIFFのJPEG変換に失敗しました。", e1);
            throw new CommandException("TIFFのJPEG変換に失敗しました。", e1);
        }
        Logger.debug("TiffToJpegCommand execute(File tiffFile)end ----------------------");

        return absolutePath;
    }

}