package jp.co.isid.elportal.fileconvert.command;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.Format;
import java.text.MessageFormat;
import java.util.List;

import jp.co.isid.elportal.base.config.ParameterCatalog;
import jp.co.isid.elportal.base.log.Logger;
import jp.co.isid.elportal.fileconvert.service.convert.ArticleDataPrintVO;
import jp.co.isid.elportal.fileconvert.service.convert.ArticleDetailVO;
import jp.co.isid.elportal.fileconvert.service.convert.ArticleOrgTextVO;
import jp.co.isid.elportal.fileconvert.service.convert.ArticleVO;
import jp.co.isid.elportal.fileconvert.service.convert.OrgTextVO;
import jp.co.isid.elportal.fileconvert.util.ArtclHdlnListPrntUtility;
import jp.co.isid.elportal.fileconvert.util.StringUtil;

import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.ExceptionConverter;
import com.itextpdf.text.Font;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.ColumnText;
import com.itextpdf.text.pdf.PdfContentByte;
import com.itextpdf.text.pdf.PdfImportedPage;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfPageEventHelper;
import com.itextpdf.text.pdf.PdfReader;
import com.itextpdf.text.pdf.PdfTemplate;
import com.itextpdf.text.pdf.PdfWriter;



/**
 * イメージファイルをPDFファイルに変換後、結合するコマンドクラスです。
 *
 *
 */
public class UnionCommand implements Command<UnionCommandParameter, File> {

    /** 縮小率のY座標の位置 **/
    //72(dpi)/2.54=28(pixel/cm) 0.9(cm)*28(pixel/cm)=25.2pixel 25.2pixel+(6point:4.5pixel⇒縮小率の1行分) + (調整)
    //static final float REDUCATION_RATIO_Y_POSITION = new BigDecimal("38").floatValue();
    //static final float REDUCATION_RATIO_Y_POSITION = new BigDecimal("45.5").floatValue();
    static final float REDUCATION_RATIO_Y_POSITION = new BigDecimal("32.5").floatValue();

    /** 注記のY座標の位置 **/
    static final float NOTE_ALL_Y_POSITION = REDUCATION_RATIO_Y_POSITION + 22f;

    /** 「※無断複製・転載禁止」のY座標の位置 */
    static final float COPYRIGHT_Y_POSITION = new BigDecimal("806").floatValue();

    /** スケール **/
    static final int SCALE = 4;

    /** マージントップ **/
    static final float ARTICLE_MARGIN_T = new BigDecimal("72").divide(new BigDecimal("2.54"), SCALE, BigDecimal.ROUND_HALF_DOWN).multiply(new BigDecimal("0.5")).floatValue();

    /** 左マージン **/
    static final float ARTICLE_MARGIN_L = new BigDecimal("72").divide(new BigDecimal("2.54"), SCALE, BigDecimal.ROUND_HALF_DOWN).multiply(new BigDecimal("1.0")).floatValue();

    /** 左右マージン **/
    static final float ARTICLE_MARGIN_LR = ARTICLE_MARGIN_L * 2;

//  /* 新聞書誌情報 */
//  private static final String ARTICLE_NWSPAPER_MSG = ParameterCatalog.instance().getParameter("FileConvertParam", "articleNwspaperMsg");
//
//  /* 雑誌書誌情報 */
//  private static final String ARTICLE_MAGAZINE_MSG = ParameterCatalog.instance().getParameter("FileConvertParam", "articleMagazineMsg");

    /** 雑誌書誌 **/
    private static final String ARTICLE_MSG = ParameterCatalog.instance().getParameter("FileConvertParam", "articleMsg");

    /** 雑誌書誌(MC) **/
    private static final String ARTICLE_MSG_MC = ParameterCatalog.instance().getParameter("FileConvertParam", "articleMsgMc");

    /** MSゴシックフォント **/
    private static final String FONT_NAME_GOTHIC = ParameterCatalog.instance().getParameter("FileConvertParam", "fontNameGothic");

    /** MS明朝フォント **/
    private static final String FONT_NAME_MINCHO = ParameterCatalog.instance().getParameter("FileConvertParam", "fontNameMincho");

    /** フッターの日時の前の文字：印刷 **/
    private static final String PRINT_SECTION_TITLE = ParameterCatalog.instance().getParameter("FileConvertParam", "printSectionTitle");

    /** フッターの日時の前の文字：表示 **/
    private static final String NON_PRINT_SECTION_TITLE = ParameterCatalog.instance().getParameter("FileConvertParam", "nonPrintSectionTitle");

    /** フッター最大サイズ **/
    private static final String FOOTER_MAX_SIZE = ParameterCatalog.instance().getParameter("FileConvertParam", "footerMaxSize");

    /** フッター最大サイズ(MC) **/
    private static final String FOOTER_MAX_SIZE_MC = ParameterCatalog.instance().getParameter("FileConvertParam", "footerMaxSizeMc");

    /** 空のPDFファイル **/
    private static final String EMPTY_PDF_FILE_PATH = ParameterCatalog.instance().getParameter("FileConvertParam", "emptyPdfFilePath");

//  /* 新聞書誌情報フォーマット */
//  private Format articleNwspaperMsg2Fmt = new MessageFormat(ARTICLE_NWSPAPER_MSG);
//
//  /* 雑誌書誌情報フォーマット */
//  private Format articleMagazineMsg2Fmt = new MessageFormat(ARTICLE_MAGAZINE_MSG);

    /** 書誌フォーマット **/
    private Format _articleMsgFmt = new MessageFormat(ARTICLE_MSG);

    /** 書誌フォーマット(MC) **/
    private Format _articleMsgMcFmt = new MessageFormat(ARTICLE_MSG_MC);



    /** MSゴシック 8p bold **/
    @SuppressWarnings("unused")
    private Font _paperNameFont = null;

    /** MS明朝 8p bold **/
    private Font _bibliographicTitleFont = null;

    /** MS明朝 6p **/
    private Font _noteAllFont = null;

    /** MS明朝 7p bold **/
    @SuppressWarnings("unused")
    private Font _printInfoFont = null;


    /** 画像種別：JPEG **/
    private static final String IMG_TYPE_J = "5";
    /** 画像種別：PNG **/
    private static final String IMG_TYPE_P = "6";


    /**
     * デフォルトコンストラクタ
     */
    public UnionCommand() {
    }

    /**
     * イメージファイルをPDFファイルに変換後、結合します。<br>
     * 引数のUnionCommanParameterから、記事原文オブジェクトとアウトプットファイル名を取得し<br>
     * 結合してファイルを生成します。
     *
     * @param arg 記事原文オブジェクトとアウトプットファイル名からなるオブジェクト
     * @return 変換後のファイル
     * @throws CommandException 変換に失敗した際にthrowされる例外
     */
    public File execute(UnionCommandParameter arg) throws CommandException {

        //保存フォルダはこの時点で作成されている。
        //yyyymm/ユニークコード＋種別＋イメージNoが出力先に指定されるので
        //yyyymmフォルダをここで作成する。
        File output = arg.getOutputFile();
        File parentDir = new File(output.getParent());
        parentDir.mkdirs();

        return union(arg.getOutputFile(), arg.getInputFiles());
    }

    /**
     * 見出し一覧を出力します
     *
     * @param doc PDFドキュメント
     * @param vo 記事VO
     * @param event フッター
     * @throws IOException エラー発生時
     * @throws DocumentException エラー発生時
     */
    private void outputHeadline(Document doc, ArticleOrgTextVO vo, TableFooter event) throws IOException, DocumentException {

        // データがなければ終了
        if (vo == null) {
            return;
        }

        // 見出し出力しなければ終了
        if (!vo.getHeadlineFlg()) {
            return;
        }

        // 見出しデータがなければ終了
        ArticleDataPrintVO printVo = vo.getArticleDataPrintVO();
        if (printVo == null) {
            return;
        }

        List<ArticleDetailVO> voList = printVo.getAcDetail();
        if (voList == null || voList.isEmpty()) {
            return;
        }

        // 線出力フラグ
        event.setOutputLineFlg(true);
        // ２ページ目以降のマージン
        doc.setMargins(20, 20, 45, 60);

        // 見出し件数を再計算（原文イメージがない記事を除いた件数にする）
        int acDataCount = ArtclHdlnListPrntUtility.getAcDataCount(vo.getOrgnFlg(), vo.getArticles(), printVo.getAcDetail(), printVo.getAcDataCount());
        printVo.setAcDataCount(acDataCount);

        // ボディヘッダー部の出力
//      doc.add(ArtclHdlnListPrntUtility.getLine());
        doc.add(ArtclHdlnListPrntUtility.getBodyHeader(printVo));
//      doc.add(ArtclHdlnListPrntUtility.getDottedLine());

        int index = 1;
        for (ArticleDetailVO detailVo : voList) {
            if (ArtclHdlnListPrntUtility.isOutputAcno(vo.getOrgnFlg(), vo.getArticles(), detailVo)) {
                detailVo.setListNo(index);

                doc.add(ArtclHdlnListPrntUtility.getBodyRow(detailVo));
                doc.add(ArtclHdlnListPrntUtility.getDottedLine());

                index++;
            }
        }

        // 原文出力するなら改ページ
        if (vo.getOrgnFlg()) {
            // マージン
            doc.setMargins(50, 50, 50, 50);
            // 改ページ
            doc.newPage();
            // 線出力フラグ
            event.setOutputLineFlg(false);
        }
    }

    /**
     * イメージファイルをPDFファイルに変換後、結合します。<br>
     * 記事原文オブジェクトから結合するＰＤＦファイルの情報と、書誌情報を取得します。<br>
     * アウトプットファイルを生成し返します。
     *
     * @param output 変換後のファイル
     * @param args   記事原文オブジェクト
     * @return 変換後のファイル
     * @throws CommandException エラー発生時
     */
    private File union(File output, ArticleOrgTextVO args) throws CommandException {

        FileOutputStream fos = null;
        try {
            PdfReader pdfReader = null;
            ArticleVO article = null;
            OrgTextVO orgText = null;

            File file = null;

            ColumnText ctNoteAll = null;
            ColumnText ctTitle = null;

            Phrase phNoteAll = null;
            Phrase phTitle = null;

            PdfPCell cellPrintAttr = null;
            PdfPCell cellLogo = null;

            //PdfPCell cellPrintInfo = null;
            //PdfPCell cellReductionRatio = null;
            //PdfPCell cellAllRightReserve = null;

            Document doc = null;
            if (args.getHeadlineFlg()) {
                doc = new Document(PageSize.A4, 20, 20, 10, 60);
            } else {
                doc = new Document(PageSize.A4, 50, 50, 50, 50);
            }

Logger.debug("変換後のファイルパス::" + output.getAbsolutePath());
            fos = new FileOutputStream(new File(output.getAbsolutePath()));

            PdfWriter writer = PdfWriter.getInstance(doc, fos);

            TableFooter event = new TableFooter(args);
            writer.setPageEvent(event);

            // MCは権限を設定しない
            if (!args.getMcFlg()) {
                int allowedPermission = 0;

                if (args.getSaveFlg()) {
                    allowedPermission = PdfWriter.ALLOW_COPY;
                }
                if (args.getCanPrint()) {
                    allowedPermission = allowedPermission | PdfWriter.ALLOW_PRINTING;
                }

                writer.setEncryption(null, null, allowedPermission, PdfWriter.STANDARD_ENCRYPTION_128);
            }

            BaseFont bfMincho = BaseFont.createFont(FONT_NAME_MINCHO, BaseFont.IDENTITY_H, BaseFont.EMBEDDED);
            BaseFont bfGothic = BaseFont.createFont(FONT_NAME_GOTHIC, BaseFont.IDENTITY_H, BaseFont.EMBEDDED);

            _paperNameFont = new Font(bfGothic, 8f, Font.BOLD);
            _bibliographicTitleFont = new Font(bfMincho, 8f, Font.BOLD);
            _noteAllFont = new Font(bfMincho, 6f);
            _printInfoFont = new Font(bfMincho, 7f, Font.BOLD);

            PdfPTable hTable = null;
            //PdfPTable fTable = null;

            PdfImportedPage ip = null;
            PdfContentByte pcb = null;

            doc.open();

            if (args.getMultiPageFlg()) {
                writer.addJavaScript("this.print(true)");
            }

            // 見出し一覧出力
            outputHeadline(doc, args, event);

            if (args.getOrgnFlg()) {
                float[] tableWidths = null;
                if (args.getMcFlg()) {
                    tableWidths = new float[]{400f, 150f};
                } else {
                    tableWidths = new float[]{10f, 1f};
                }

                int articleCount = getArticleCount(args);
                int pageCount = 1;
                for (int i = 0; i < args.getArticles().length; i++) {

                    article = args.getArticles()[i];
//                  if (!article.isOutputAcno()) {
//                        Logger.debug("AC番号：" + article.getAcNo() + " 全部原文ファイルなし");
//                        continue;
//                  }
                    if (!isOutputAcno(article, args.getMcFlg())) {
                        Logger.debug("AC番号：" + article.getAcNo() + " 全部原文ファイルなし");
                        continue;
                    }

                    //記事タイトル
                    String acTitle = convertNullToKara(article.getAcTitle());

                    int orgnCount = 1;
                    for (int c = 0; c < article.getOrgTexts().length; c++) {

                        // 原文ファイルがなかったら印刷しない
                        if (!article.isOutputOrg(c)) {
                            Logger.debug("AC番号：" + article.getAcNo() + " インデックス：" + c + " 原文ファイルなし");
                            continue;
                        }

                        //記事属性
                        String articleAttribute = getArticleAttribute(article, pageCount, articleCount, orgnCount, args.getMcFlg()); //記事属性編集

                        orgText = article.getOrgTexts()[c];
                        file = orgText.getOrgText();

                        //拡張子がpdfの場合変換しない。pdf以外の場合、pdfに変換する
                        long pdfStartTime = System.currentTimeMillis();
                        Logger.info("PDFファイルを読み込み開始　ファイル名；" + file.getAbsolutePath());
                        pdfReader = new PdfReader(createPdfReaderPath(file));
                        Logger.info("PDFファイルを読み込み完了　経過時間：" + (System.currentTimeMillis() - pdfStartTime) + "ms");

                        ip = writer.getImportedPage(pdfReader, 1);
                        pcb = writer.getDirectContent();
                        pcb.addTemplate(ip, 0f, 0f);
                        pcb.setFontAndSize(bfMincho, 12f);

                        //ヘッダーテーブル（書誌情報、■ＥＬＮＥＴ■）
                        hTable = new PdfPTable(2);
                        hTable.setWidths(tableWidths);
                        hTable.setTotalWidth(doc.getPageSize().getWidth() - ARTICLE_MARGIN_LR);
                        hTable.setLockedWidth(true);
                        hTable.getDefaultCell().setFixedHeight(100f);

                        //書誌情報
                        cellPrintAttr = new PdfPCell(new Paragraph(articleAttribute, _bibliographicTitleFont));
                        cellPrintAttr.setPadding(0f);
                        cellPrintAttr.setBorder(Rectangle.NO_BORDER);
                        cellPrintAttr.setHorizontalAlignment(Element.ALIGN_LEFT);
                        hTable.addCell(cellPrintAttr);

                        //■ＥＬＮＥＴ■ロゴ
                        if (args.getMcFlg()) {
                            // MCの場合
                            String page = _articleMsgMcFmt.format(new String[]{String.valueOf(orgnCount), String.valueOf(article.getOutputOrgCount())});
                            cellLogo = new PdfPCell(new Paragraph(page, _bibliographicTitleFont));
                            cellLogo.setPadding(0f);
                            cellLogo.setPaddingRight(60f);
                        } else {
                            // 通常の場合
                            cellLogo = new PdfPCell(new Paragraph());
                            cellLogo.setPadding(0f);
                        }
                        cellLogo.setBorder(Rectangle.NO_BORDER);
                        cellLogo.setHorizontalAlignment(Element.ALIGN_RIGHT);
                        hTable.addCell(cellLogo);


                        hTable.writeSelectedRows(0, -1, ARTICLE_MARGIN_L, doc.getPageSize().getHeight() - ARTICLE_MARGIN_T, pcb);


                        //見出し
                        ctTitle = new ColumnText(pcb);
                        ctTitle.setSimpleColumn(ARTICLE_MARGIN_L, doc.getPageSize().getHeight() - 24f, doc.getPageSize().getWidth() - ARTICLE_MARGIN_L - 130, doc.getPageSize().getHeight() - 68f);
                        phTitle = Phrase.getInstance(0, acTitle, _bibliographicTitleFont);
                        ctTitle.addText(phTitle);
                        ctTitle.setLeading(11f);
                        ctTitle.go();

                        //注記
                        ctNoteAll = new ColumnText(pcb);
                        ctNoteAll.setSimpleColumn(ARTICLE_MARGIN_L, NOTE_ALL_Y_POSITION, doc.getPageSize().getWidth() - ARTICLE_MARGIN_L, REDUCATION_RATIO_Y_POSITION);
                        phNoteAll = Phrase.getInstance(0, article.getNoteAll(), _noteAllFont);
                        ctNoteAll.addText(phNoteAll);
                        ctNoteAll.setLeading(10f);
                        ctNoteAll.go();

                        doc.newPage();

                        orgnCount++;
                    }

                    pageCount++;
                }

                // 見出しなし、原文ありで原文印刷する記事がなかったら空のページを出す
                if (!args.getHeadlineFlg() &&  articleCount == 0) {
                    Logger.debug("空ファイルを出力");
                    pdfReader = new PdfReader(new File(EMPTY_PDF_FILE_PATH).getAbsolutePath());
                    ip = writer.getImportedPage(pdfReader, 1);
                    pcb = writer.getDirectContent();
                    pcb.addTemplate(ip, 0f, 0f);
                    doc.newPage();
                }
            }
            long startTime = System.currentTimeMillis();
            Logger.info("PDF出力を開始します");
            doc.close();
            Logger.info("PDF出力を終了します　経過時間：" + (System.currentTimeMillis() - startTime) + "ms　出力ファイル：" + output.getAbsolutePath());
        } catch (IOException e) {
            Logger.error("PDF結合に失敗しました。", e);
            throw new CommandException("PDF結合に失敗しました。", e);
        } catch (DocumentException e) {
            Logger.error("ドキュメントを開くことに失敗しました", e);
            throw new CommandException("ドキュメントを開くことに失敗しました。", e);
        } finally {
            if (fos != null) {
                try {
                    fos.close();
                } catch (IOException e) {
                }
            }
        }


        return new File(output.getAbsolutePath());
    }

    /**
     * 書誌情報を返します。
     *
     * @param article          記事データ
     * @param articlePage      記事ページ
     * @param articleTotalPage 記事ページ総数
     * @param orgTextPage      原文ページ
     * @param mcFlg            MCフラグ
     * @return 書誌情報
     */
    private String getArticleAttribute(ArticleVO article, int articlePage, int articleTotalPage, int orgTextPage, boolean mcFlg) {
        String result = "";
        StringBuffer str1 = new StringBuffer();
        StringBuffer str2 = new StringBuffer();
        String mediaNm = convertNullToKara(article.getMediaNm());
        String issueDate = convertNullToKara(article.getIssueDate());
        String moveType = convertNullToKara(article.getMoveType());
//      String frameNm = convertNullToKara(article.getFrameNm());
        String frameNo = convertNullToKara(article.getFrameNo());

        String tome = convertNullToKara(article.getTome());
        String issue = convertNullToKara(article.getIssue());
        String pages = convertNullToKara(article.getPages());

        String space = "　";
        String articleAttribute = "";

        str1.append(moveType).append(frameNo);
        str2.append(tome).append(issue).append(pages);

//      if(!StringUtil.isBlank(str1.toString())) {
//          //新聞
//          String[] articleNwspaper = {mediaNm, issueDate, moveType, frameNo, Integer.toString(articlePage), Integer.toString(articleTotalPage), String.valueOf(orgTextPage), String.valueOf(article.getOrgTexts().length)};
//          result = articleNwspaperMsg2Fmt.format(articleNwspaper);
//      } else if(!StringUtil.isBlank(str2.toString())){
//          //雑誌
//          String[] articleMagazine = {mediaNm, tome, issue, pages, issueDate, Integer.toString(articlePage), Integer.toString(articleTotalPage), String.valueOf(orgTextPage), String.valueOf(article.getOrgTexts().length)};
//          result = articleMagazineMsg2Fmt.format(articleMagazine);
//      }

        if (!StringUtil.isBlank(str1.toString())) {
            //新聞
            StringBuffer articleNwspaper = new StringBuffer();
            if (!StringUtil.isBlank(mediaNm)) {
                articleNwspaper.append("◎").append(mediaNm);
            }
            if (!StringUtil.isBlank(issueDate)) {
                articleNwspaper.append(space).append(issueDate);
            }
            if (!StringUtil.isBlank(moveType)) {
                articleNwspaper.append(space).append(moveType);
            }
            if (!StringUtil.isBlank(frameNo)) {
                articleNwspaper.append(frameNo).append("面");
            }
            articleAttribute = articleNwspaper.toString();
        } else if (!StringUtil.isBlank(str2.toString())) {
            //雑誌
            StringBuffer articleMagazine = new StringBuffer();
            if (!StringUtil.isBlank(mediaNm)) {
                articleMagazine.append("◎").append(mediaNm);
            }
            if (!StringUtil.isBlank(tome)) {
                articleMagazine.append(space).append(tome).append("巻");
            }
            if (!StringUtil.isBlank(issue)) {
                articleMagazine.append(space).append(issue).append("号");
            }
            if (!StringUtil.isBlank(pages)) {
                articleMagazine.append(space).append(pages).append("頁");
            }
            if (!StringUtil.isBlank(issueDate)) {
                articleMagazine.append(space).append(issueDate);
            }
            articleAttribute = articleMagazine.toString();
        }

        if (mcFlg) {
            // MCの場合
            result = articleAttribute;
        } else {
            // 通常の場合
            String[] articleAttributeData = new String[]{
                    articleAttribute,
                    Integer.toString(articlePage),
                    Integer.toString(articleTotalPage),
                    String.valueOf(orgTextPage),
                    String.valueOf(article.getOutputOrgCount())
            };

            result = _articleMsgFmt.format(articleAttributeData);
        }

        return result;
    }

    /**
     * 原文出力記事数を返却します
     *
     * @param args 原文VO
     * @return int 原文出力記事数
     */
    private int getArticleCount(ArticleOrgTextVO args) {
        if (args == null) {
            return 0;
        }

        ArticleVO[] array = args.getArticles();
        if (array == null || array.length == 0) {
            return 0;
        }

        int count = 0;
        for (ArticleVO vo : array) {
            if (isOutputAcno(vo, args.getMcFlg())) {
                count++;
            }
        }

        return count;
    }

    /**
     * 原文出力するかどうか（AC番号単位）
     *
     * @param vo 原文VO
     * @param mcFlg MCフラグ
     * @return boolean true - する、false - しない
     */
    private boolean isOutputAcno(ArticleVO vo, boolean mcFlg) {
        boolean output = true;
        if (mcFlg) {
            // MCの場合は一枚でも原文取得できなかったら出力しない
            int size = vo.getOrgTexts().length;
            for (int i = 0; i < size; i++) {
                if (!vo.isOutputOrg(i)) {
                    output = false;
                    break;
                }
            }
        } else {
            // 通常の場合は一枚でも原文取得できたら出力する
            output = vo.isOutputAcno();
        }

        return output;
    }

    /**
     * 渡された文字列がnullの場合、空文字を返します。
     *
     * @param str 文字列
     * @return String 変換した文字列
     */
    private String convertNullToKara(String str) {
        String result = "";

        if (!StringUtil.isBlank(str)) {
            result = str;
        }
        return result;
    }

    /**
     *
     * ファイルがイメージの場合、PDFに変換します。
     *
     * @param file 変換元のファイル
     * @return 変換後のファイルの保存パス
     * @throws CommandException エラー発生時
     */
    private String createPdfReaderPath(File file) throws CommandException {

        String fileName = file.getName().toLowerCase();

        // 拡張子がpdfの場合
        if (fileName.endsWith(".pdf")) {
            return file.getAbsolutePath();
        }

        String fileType = fileName.substring(13, 14);
        if (IMG_TYPE_J.equals(fileType) || IMG_TYPE_P.equals(fileType)) {
            // ファイル区分がJPEGまたはPNGの場合
            // JPEGまたはPNGをPDFに変換してPDFのパスを返却する
            JpgPngToPdfCommand command = new JpgPngToPdfCommand();
            File tempFile = command.execute(file, file.getAbsolutePath() + ".pdf");
            return tempFile.getAbsolutePath();
        } else {
            // それ以外の場合はtiffの場合
            // TIFFをPDFに変換してPDFのパスを返却する
            TiffToPdfCommand command = new TiffToPdfCommand();
            File tempFile = command.execute(file, file.getAbsolutePath() + ".pdf");
            return tempFile.getAbsolutePath();
        }
    }

    /**
     * Inner class to add a table as header.
     */
    class TableFooter extends PdfPageEventHelper {

        /** The template with the total number of pages. */
        private PdfTemplate _total;

        /** PDF出力情報 **/
        private ArticleOrgTextVO _vo;
        /** 線出力フラグ **/
        private boolean _outputLineFlg = false;

        /** ヘッダーメッセージ **/
        private static final String HEADER_MSG = "■ＥＬＮＥＴ■";
        /** コピーライトメッセージ **/
        private static final String COPYRIGHT_MSG = "※無断複製・転載禁止";
        /** マージン（横） **/
        private static final float MERGIN_WIDTH = 20;
        /** マージン（上） **/
        private static final float MERGIN_TOP = 20;
        /** マージン（下） **/
        private static final float MERGIN_BOTTOM = 25;

        /**
         * コンストラクター
         *
         * @param vo 記事原文VO
         */
        public TableFooter(ArticleOrgTextVO vo) {
            this._vo = vo;
        }

        /**
         * 線出力フラグを設定します
         *
         * @param flg 線出力フラグ
         */
        public void setOutputLineFlg(boolean flg) {
            this._outputLineFlg = flg;
        }

        /**
         * 線出力をするかどうか
         *
         * @return boolean 線出力をするかどうか
         */
        public boolean isOutputLine() {
            return this._outputLineFlg;
        }

        /**
         * Creates the PdfTemplate that will hold the total number of pages.
         * @see com.itextpdf.text.pdf.PdfPageEventHelper#onOpenDocument(
         *      com.itextpdf.text.pdf.PdfWriter, com.itextpdf.text.Document)
         */
        @Override
        public void onOpenDocument(PdfWriter writer, Document document) {
            try {
                PdfContentByte pcb = writer.getDirectContent();
                _total = pcb.createTemplate(30, 16);
            } catch (Exception e) {
                Logger.error("ドキュメントを開くことに失敗しました：onOpenDocument");
                throw new ExceptionConverter(e);
            }
        }

        /**
         * Adds a header to every page
         * ※newPageした状態でこのメソッド実装してしまうと空ページができてしまうもよう
         * @see com.itextpdf.text.pdf.PdfPageEventHelper#onStartPage(
         *      com.itextpdf.text.pdf.PdfWriter, com.itextpdf.text.Document)
         */
//        public void onStartPage(PdfWriter writer, Document document) {
//            try {
//                PdfContentByte pcb = writer.getDirectContent();
//                BaseFont bf = BaseFont.createFont(FONT_NAME_GOTHIC, BaseFont.IDENTITY_H, BaseFont.EMBEDDED);
//                Font bold = new Font(bf, 8f, Font.BOLD);
//
//                // ヘッダーメッセージ
//                ColumnText.showTextAligned(pcb, PdfContentByte.ALIGN_RIGHT, new Phrase(HEADER_MSG, bold),
//                      document.getPageSize().getWidth() - MERGIN_WIDTH,
//                      document.getPageSize().getHeight() - MERGIN_TOP,
//                      0);
//
////                pcb.beginText();
////                pcb.setFontAndSize(bf, 11);
////                pcb.showTextAligned(PdfContentByte.ALIGN_RIGHT, HEADER_MSG,
////                        document.getPageSize().getWidth() - MERGIN_WIDTH,
////                        document.getPageSize().getHeight() - MERGIN_TOP,
////                        0);
////                pcb.endText();
//
//                // 線を引く
//                float linePositionY = document.getPageSize().getHeight() - MERGIN_TOP - 20;
//                pcb.setLineWidth(2.0f);
//                pcb.setColorStroke(new BaseColor(0xCC, 0xCC, 0xCC));
//                pcb.moveTo(MERGIN_WIDTH, linePositionY);
//                pcb.lineTo(document.getPageSize().getWidth() - MERGIN_WIDTH, linePositionY);
//                pcb.stroke();
//
//                writer.getDirectContent().addTemplate(total, document.getPageSize().getWidth()/2 + 15, 23);
//            } catch(DocumentException de) {
//                Logger.error("文字の書き出しに失敗しました。");
//                throw new ExceptionConverter(de);
//            } catch(IOException ie) {
//                Logger.error("フォントが見つかりません。");
//                throw new ExceptionConverter(ie);
//            }
//
//        }

        /**
         * Adds a header to every page
         * @see com.itextpdf.text.pdf.PdfPageEventHelper#onEndPage(
         *      com.itextpdf.text.pdf.PdfWriter, com.itextpdf.text.Document)
         */
        @Override
        public void onEndPage(PdfWriter writer, Document document) {
            try {
                PdfContentByte pcb = writer.getDirectContent();

                // 色
                BaseColor black = new BaseColor(0x00, 0x00, 0x00);
                BaseColor red = new BaseColor(0xff, 0x00, 0x00);

                // フォント（ゴシック）
                BaseFont bfGothic = BaseFont.createFont(FONT_NAME_GOTHIC, BaseFont.IDENTITY_H, BaseFont.EMBEDDED);
                Font boldH = new Font(bfGothic, 8f, Font.BOLD, black);
                Font boldR = new Font(bfGothic, 12f, Font.BOLD, red);

                // フォント（明朝）
                BaseFont bfMincho = BaseFont.createFont(FONT_NAME_MINCHO, BaseFont.IDENTITY_H, BaseFont.EMBEDDED);
                Font boldP = new Font(bfMincho, 7f, Font.BOLD, black);
                Font boldN = new Font(bfMincho, 8f, Font.BOLD, black);

                // ヘッダーメッセージ
                ColumnText.showTextAligned(pcb, PdfContentByte.ALIGN_RIGHT, new Phrase(HEADER_MSG, boldH),
                        document.getPageSize().getWidth() - MERGIN_WIDTH,
                        document.getPageSize().getHeight() - MERGIN_TOP,
                        0);
                ColumnText.showTextAligned(pcb, PdfContentByte.ALIGN_RIGHT, new Phrase(COPYRIGHT_MSG, boldR),
                        document.getPageSize().getWidth() - MERGIN_WIDTH,
                        COPYRIGHT_Y_POSITION ,
                        0);

                // 見出し一覧は線を引く
                if (isOutputLine()) {
                    // 線を引く（ヘッダー）
                    float linePositionY = document.getPageSize().getHeight() - MERGIN_TOP - 20;
                    pcb.setLineWidth(2.0f);
                    pcb.setColorStroke(new BaseColor(0xCC, 0xCC, 0xCC));
                    pcb.moveTo(MERGIN_WIDTH, linePositionY);
                    pcb.lineTo(document.getPageSize().getWidth() - MERGIN_WIDTH, linePositionY);
                    pcb.stroke();

                    // 線を引く（フッター）
                    pcb.setLineWidth(2.0f);
                    pcb.setColorStroke(new BaseColor(0xCC, 0xCC, 0xCC));
                    pcb.moveTo(MERGIN_WIDTH, MERGIN_BOTTOM + 30);
                    pcb.lineTo(document.getPageSize().getWidth() - MERGIN_WIDTH, MERGIN_BOTTOM + 30);
                    pcb.stroke();
                }

                // フッターメッセージ
                ColumnText.showTextAligned(pcb, PdfContentByte.ALIGN_LEFT, new Phrase(getPrintInfo(), boldP), MERGIN_WIDTH, MERGIN_BOTTOM , 0);
                if (!_vo.getMcFlg()) {
                    ColumnText.showTextAligned(pcb, PdfContentByte.ALIGN_CENTER, new Phrase(writer.getPageNumber() + "/", boldN), document.getPageSize().getWidth() / 2, MERGIN_BOTTOM - 10, 0);
                }
                writer.getDirectContent().addTemplate(_total, document.getPageSize().getWidth() / 2 + 15, MERGIN_BOTTOM - 10);

//                pcb.beginText();
//                pcb.setFontAndSize(bf, 8);
//                pcb.showTextAligned(PdfContentByte.ALIGN_LEFT, getPrintInfo(), MERGIN_WIDTH, MERGIN_BOTTOM + 10, 0);
//                pcb.showTextAligned(PdfContentByte.ALIGN_RIGHT, FOOTER_MSG, document.getPageSize().getWidth() - MERGIN_WIDTH, MERGIN_BOTTOM + 10, 0);
//                pcb.setFontAndSize(bf, 10);
//                pcb.showTextAligned(PdfContentByte.ALIGN_CENTER, writer.getPageNumber() + "/", document.getPageSize().getWidth()/2, MERGIN_BOTTOM, 0);
//                pcb.endText();

//                writer.getDirectContent().addTemplate(total, document.getPageSize().getWidth()/2 + 15, MERGIN_BOTTOM - 2);
            } catch (DocumentException de) {
                Logger.error("文字の書き出しに失敗しました：onEndPage");
                throw new ExceptionConverter(de);
            } catch (IOException ie) {
                Logger.error("フォントが見つかりません：onEndPage");
                throw new ExceptionConverter(ie);
            }
        }

        /**
         * Fills out the total number of pages before the document is closed.
         * @see com.itextpdf.text.pdf.PdfPageEventHelper#onCloseDocument(
         *      com.itextpdf.text.pdf.PdfWriter, com.itextpdf.text.Document)
         */
        @Override
        public void onCloseDocument(PdfWriter writer, Document document) {

            try {
                BaseFont bf = BaseFont.createFont(FONT_NAME_MINCHO, BaseFont.IDENTITY_H, BaseFont.EMBEDDED);
                Font font = new Font(bf, 8f, Font.BOLD, new BaseColor(0x00, 0x00, 0x00));

                if (!_vo.getMcFlg()) {
                    ColumnText.showTextAligned(_total, Element.ALIGN_CENTER,
                            new Phrase(String.valueOf(writer.getPageNumber() - 1), font),
                            7, 0, 0);
                }

            } catch (DocumentException de) {
                Logger.error("文字の書き出しに失敗しました：onCloseDocument");
                throw new ExceptionConverter(de);
            } catch (IOException ie) {
                Logger.error("フォントが見つかりません：onCloseDocument");
                throw new ExceptionConverter(ie);
            }
        }

        /**
         * 印刷情報を返却します
         *
         * @return String 印刷情報
         */
        private String getPrintInfo() {
            if (_vo == null) {
                return "";
            }

            if (_vo.getMcFlg()) {
                // MCの場合
                return MessageFormat.format("日時：{0} サーバーID：{1} MCユーザID：{2}",
                        toBlank(_vo.getPrintDate()),
                        toBlank(_vo.getDmsId()),
                        cutString(_vo.getMcUserId()));
            } else {
                // 通常の場合
                StringBuffer str = new StringBuffer((_vo.getCanPrint() ? PRINT_SECTION_TITLE : NON_PRINT_SECTION_TITLE));
                str.append(MessageFormat.format("日時：{0} ユーザ：{1} {2} 様", toBlank(_vo.getPrintDate()), toBlank(_vo.getPrintUserId()), cutString(_vo.getPrintUser())));
                return str.toString();
            }
        }

        /**
         * 最大文字数でカットします
         *
         * @param value 文字列
         * @return String カットした文字列
         */
        private String cutString(String value) {
            if (value == null || "".equals(value)) {
                return "";
            }

            int maxLength = 0;
            if (_vo.getMcFlg()) {
                maxLength = parseInt(FOOTER_MAX_SIZE_MC);
            } else {
                maxLength = parseInt(FOOTER_MAX_SIZE);
            }

            // 最大文字数未設定ならそのまま
            if (maxLength <= 0) {
                return value;
            }

            // 最大文字数以下ならそのまま
            if (value.length() <= maxLength) {
                return value;
            }

            // 最大文字数超えてたら切る
            return value.substring(0, maxLength);
        }

        /**
         * 文字列を数値に変換する
         *
         * @param value 文字列
         * @return int 数値
         */
        private int parseInt(String value) {
            if (value == null) {
                return 0;
            }

            try {
                return Integer.parseInt(value);
            } catch (Exception e) {
                return 0;
            }
        }

        /**
         * NULLをブランクに変換
         *
         * @param value 変換する文字列
         * @return String 変換した文字列
         */
        private String toBlank(String value) {
            if (value == null) {
                return "";
            }

            return value;
        }
    }

}
