package jp.co.isid.elportal.fileconvert.service.convert;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;

import jp.co.isid.elportal.base.cifs.CifsAccessor;
import jp.co.isid.elportal.base.cifs.CifsAccessorFactory;
import jp.co.isid.elportal.base.cifs.exception.CifsApplicationException;
import jp.co.isid.elportal.base.config.ParameterCatalog;
import jp.co.isid.elportal.base.log.Logger;
import jp.co.isid.elportal.base.util.ServiceStatusCheckUtil;
import jp.co.isid.elportal.fileconvert.command.CommandException;
import jp.co.isid.elportal.fileconvert.command.UnionCommand;
import jp.co.isid.elportal.fileconvert.command.UnionCommandParameter;
import jp.co.isid.elportal.fileconvert.consts.CommonConstants.CifsType;
import jp.co.isid.elportal.fileconvert.util.ArtclHdlnListPrntUtility;
import jp.co.isid.elportal.fileconvert.util.FileUtil;
import jp.co.isid.elportal.fileconvert.util.StringUtil;
import jp.co.isid.soa.framework.service.exception.ExceptionHandler;
import jp.co.isid.soa.framework.service.exception.ServiceException;

import org.apache.commons.io.IOUtils;

/**
 *
 * Webアプリケーションから呼び出される、TIFFファイル変換サービスです。
 *
 *
 */
@WebService(
        serviceName = "convertService",
        targetNamespace = "http://convert.model.file_convert.elportal.isid.co.jp/"
        )
public class ConvertService {

    /** ファイル存在 **/
    private static final String FILE_EXIST = "EXIST";
    /** ファイル存在しない **/
    private static final String FILE_NOT_EXIST = "NOTEXIST";
    /** PDF結合ファイル配置ディレクトリ **/
    private static final String FILE_SAVE_DIR = ParameterCatalog.instance().getParameter("FileConvertParam", "imageWorkDir");
    /** 空のPDFファイル **/
    private static final String EMPTY_PDF_FILE_PATH = ParameterCatalog.instance().getParameter("FileConvertParam", "emptyPdfFilePath");
    /** FFSの一年越えTIFFファイル配置ディレクトリ **/
    private static final String CIFS_OVER_YEAR_FILE_DIR = ParameterCatalog.instance().getParameter("FileConvertParam", "cifsFileDir");
    /** CIFSダウンロードファイル配置ディレクトリ **/
    private static final String CIFS_DL_DIR = ParameterCatalog.instance().getParameter("FileConvertParam", "cifsDlFolder");
    /** 変換後ファイル区切り文字 **/
    private static final String RETURN_FILE_SEPARATOR = "/";
    /** 変換前ファイル区切り文字 **/
    private static final String REPLACE_FILE_SEPARATOR = "\\\\";
    /** 一時保存ディレクトリ区切り文字 **/
    private static final String DIR_NAME_SPLIT = "_";
    /** 画像種別：白黒 **/
    private static final String IMG_TYPE_WB = "0";
    /** 画像種別：カラー **/
    private static final String IMG_TYPE_C = "1";
    /** 画像種別：JPEG **/
    private static final String IMG_TYPE_J = "5";
    /** 画像種別：PNG **/
    private static final String IMG_TYPE_P = "6";

    /**
     *  絶対ファイルパスのリストをみて、それらを結合したPDFを作成し、ユーザIDを埋め込みます。
     *
     * @param atcOrgText 印刷日、ユーザID、イメージファイルの絶対パスなど。<br>
     * @return 結合したPDFファイルを返します。（MTOM）
     * @throws ServiceException 変換に失敗した場合。
     */
    @WebMethod
    @WebResult(name = "pdfFile")
    public ConvertPDFVO unionAndAddUser(
            @WebParam(name = "atcOrgText")ArticleOrgTextVO atcOrgText) throws ServiceException {


        long allStartTime = System.currentTimeMillis();
        Logger.info("PDF作成処理を開始します");

        ConvertPDFVO cvo = new ConvertPDFVO();

        try {
            // 作業ディレクトリ名取得
            String workDirName = getWorkDirName(atcOrgText);

            ArrayList<String> resultList = new ArrayList<String>();

            File outputFile = null;
            if (atcOrgText.getOrgnFlg()) {
                // 原文出力する場合
                ArticleVO[] articles = atcOrgText.getArticles();

                if (articles == null || articles.length == 0) {
                    return cvo;
                }

                int articleCnt = articles.length;        //記事数

                for (int i = 0; i < articleCnt; i++) {

                    long startTime = System.currentTimeMillis();

                    if (articles[i].getOverYearOrgFlg()) {
                        Logger.info("FFSからファイルを取得します");
                        //1年超えのファイル→FFSから取得
                        setFilesFromFFS(articles[i], resultList, workDirName);
                        Logger.info("FFSからファイルを取得しました　経過時間：" + (System.currentTimeMillis() - startTime) + "ms");
                    } else {
                        Logger.info("設定ファイルで指定した場所からファイルを取得します");
                        setFiles(articles[i], resultList);
                        Logger.info("設定ファイルで指定した場所からファイルを取得しました　経過時間：" + (System.currentTimeMillis() - startTime) + "ms");
                    }
                }

                outputFile =
                        new File(
                                new File(makeSaveDir(workDirName)),
                                editSaveFileName(atcOrgText.getArticles()[0].getOrgTexts()[0].getOrgTextFileName()));

            } else if (atcOrgText.getHeadlineFlg()) {
                // 見出し出力のみの場合
                outputFile = File.createTempFile("PDFOUT_", ".pdf", new File (makeSaveDir(workDirName)));
            }

            UnionCommand unionCommand = new UnionCommand();

            File unionFile = unionCommand.execute(new UnionCommandParameter(atcOrgText, outputFile));

            DataHandler dh = new DataHandler(new FileDataSource(unionFile.getAbsolutePath()));
            cvo.setConvertResult((String[])resultList.toArray(new String[]{}));
            cvo.setOutputAcnoArray(getOutputAcnoArray(atcOrgText));
            cvo.setConvertPdf(dh);

        } catch (IOException e) {
            Logger.error("一時ファイル作成に失敗しました", e);
            ExceptionHandler.getInstance().handleException(e);
            throw new AssertionError();
        } catch (CommandException e) {
            ExceptionHandler.getInstance().handleException(e);
            throw new AssertionError();
        }

        Logger.info("PDF作成処理を終了します　経過時間：" + (System.currentTimeMillis() - allStartTime) + "ms");

        return cvo;
    }

    /**
     * 見出し一覧に出力したAC番号の配列を返却します
     *
     * @param vo 原文記事VO
     * @return String[] 見出し一覧に出力したAC番号の配列
     */
    private String[] getOutputAcnoArray(ArticleOrgTextVO vo) {

        // 出力データがなかったら終了
        if (vo == null) {
            return null;
        }

        // 見出し一覧を出力しなかったら終了
        if (!vo.getHeadlineFlg()) {
            return null;
        }

        // 見出しデータがなかったら終了
        ArticleDataPrintVO printVo = vo.getArticleDataPrintVO();
        if (printVo == null) {
            return null;
        }

        // 見出し一覧がなかったら終了
        List<ArticleDetailVO> detailVoList = printVo.getAcDetail();
        if (detailVoList == null || detailVoList.isEmpty()) {
            return null;
        }

        // 見出し一覧出力可否チェック
        List<String> acnoList = new ArrayList<String>();
        for (ArticleDetailVO detailVo : detailVoList) {

            // AC番号未設定なら飛ばす
            if (StringUtil.isBlank(detailVo.getAcno())) {
                continue;
            }

            // 見出し一覧に出力するAC番号ならリストに追加する
            if (ArtclHdlnListPrntUtility.isOutputAcno(vo.getOrgnFlg(), vo.getArticles(), detailVo)) {
                if (!acnoList.contains(detailVo.getAcno())) {
                    acnoList.add(detailVo.getAcno());
                }
            }
        }

        return acnoList.toArray(new String[]{});
    }

    /**
     * 1年超えのファイルについてはCIFS通信API経由でファイルを取得する。
     * 所得したファイルは原文オブジェクトにセットされる。
     *
     * @param article 記事データ
     * @param resultList 結果リスト
     * @param workDirName 作業ディレクトリ名
     */
    private void setFilesFromFFS(ArticleVO article, ArrayList<String> resultList, String workDirName) {
        Logger.debug("setFilesFromFFS(ArticleVO article, ArrayList<String> resultList)start ----------------------");

        OrgTextVO[] orgTextVoArray = article.getOrgTexts();
        File orgText = null;
        String filePath = null;

        int index = 0;
        boolean[] orgnFlgs = createArray(orgTextVoArray);

        // FFS稼働チェック
        boolean ftpAvailable = ServiceStatusCheckUtil.isFtpAvailable();
        Logger.debug("FFSが稼働しているか？::" + ftpAvailable);
        if (!ftpAvailable) {
            index = 0;
            for (OrgTextVO orgTextVo : orgTextVoArray) {
                File emptyFile = new File(EMPTY_PDF_FILE_PATH);
                orgTextVo.setOrgText(emptyFile);
                orgTextVo.setOrgTextFileName(emptyFile.getName());
                resultList.add(FILE_NOT_EXIST);

                orgnFlgs[index] = false;
                index++;
            }

            article.setOrgFlgs(orgnFlgs);
            return;
        }

        try {
            // FFSのTIFF画像ディレクトリを参照
            CifsAccessor cifs = CifsAccessorFactory.getInstance().getAccessor(CifsType.FFS_TIFF.getName());

            index = 0;
            for (OrgTextVO orgTextVo : orgTextVoArray) {

                Logger.debug("OrgTextVO.getOrgTextFileNameにファイル名が設定されているか？::" + !StringUtil.isBlank(orgTextVo.getOrgTextFileName()));
                if (!StringUtil.isBlank(orgTextVo.getOrgTextFileName())) {
                    // ファイル名がセットされている場合
                    try {

                        // FFSからのダウンロードファイル保存先ディレクトリ
                        String editSavePathForFFS = this.editSavePathForFFS(workDirName);
                        // FFSのTIFF画像ファイルパス
                        String fileName = orgTextVo.getOrgTextFileName();

                        Logger.debug("CIFS通信APIにわたす、保存先のディレクトリのパス（絶対パスで指定）::" + editSavePathForFFS);
                        orgText = new File(editSavePathForFFS, fileName);
                        filePath = orgText.getPath();

                        byte[] binary = this.download(cifs, fileName, this.isColor(fileName));
                        this.copy(binary, orgText);

                        orgTextVo.setOrgText(orgText);
                        resultList.add(FILE_EXIST);
                        Logger.debug(Thread.currentThread().getName() + " --- FFSから取得したファイル::" + filePath + ",length=" + orgText.length());

                        orgnFlgs[index] = true;
                    } catch (CifsApplicationException | IOException e) {
                        Logger.error("FFSからのファイル取得に失敗しました", e);
                        orgTextVo.setOrgText(new File(EMPTY_PDF_FILE_PATH));
                        resultList.add(FILE_NOT_EXIST);

                        orgnFlgs[index] = false;
                    }
                } else {
                    //ファイル名
                    File emptyFile = new File(EMPTY_PDF_FILE_PATH);
                    orgTextVo.setOrgText(emptyFile);
                    orgTextVo.setOrgTextFileName(emptyFile.getName());
                    resultList.add(FILE_NOT_EXIST);

                    orgnFlgs[index] = false;
                }

                index++;
            }

            article.setOrgFlgs(orgnFlgs);
        } catch (Exception e) {
            // エラーがあったら全部空のファイルにする
            Logger.error("IFSからのファイル取得に失敗しました（想定外）", e);

            for (OrgTextVO orgTextVo : orgTextVoArray) {
                File emptyFile = new File(EMPTY_PDF_FILE_PATH);
                orgTextVo.setOrgText(emptyFile);
                orgTextVo.setOrgTextFileName(emptyFile.getName());
                resultList.add(FILE_NOT_EXIST);

                orgnFlgs[index] = false;
            }
        }
        Logger.debug("setFilesFromFFS(ArticleVO article, ArrayList<String> resultList)end ----------------------");
    }

    /**
     * FFSからTIFF画像をダウンロードします
     *
     * @param cifs CIFS
     * @param fileName ダウンロードファイル名
     * @param retry ファイルが取得できなかった場合再取得するかどうか
     * @return byte[] ダウンロードファイル
     * @throws CifsApplicationException ファイルがなかった場合
     */
    private byte[] download(CifsAccessor cifs, String fileName, boolean retry) throws CifsApplicationException {
        byte[] binary = null;
        try {

            // FFSのTIFF画像ファイルパス
            String editPathForFFS = this.editPathForFFS(fileName);
            Logger.info("TIFFファイル [" + editPathForFFS + "]");
            Logger.debug("CIFS通信APIにわたす、ダウンロードしたいファイルのパス（ファイル名まで相対パスで指定）::" + editPathForFFS);

            // TIFF画像取得
           binary = cifs.getBytes(editPathForFFS);
        } catch (CifsApplicationException e) {
            // ファイルがなくてもカラーの場合は白黒の取得を試みる
            if (retry) {
                Logger.error("FFS画像ダウンロード失敗", e);

                String convertFileName = this.convertFileName(fileName);
                binary = this.download(cifs, convertFileName, isColor(convertFileName));
            } else {
                throw e;
            }
        }

        return binary;
    }

    /**
     * カラー画像ファイル名を白黒画像ファイル名に変換します
     *
     * @param fileName カラー画像ファイル名
     * @return String 白黒画像ファイル名
     */
    private String convertFileName(String fileName) {

        String fileType = fileName.substring(13, 14);
        String nextType = IMG_TYPE_WB;
        if (IMG_TYPE_J.equals(fileType)) {
            nextType = IMG_TYPE_P;
        } else if (IMG_TYPE_P.equals(fileType)) {
            nextType = IMG_TYPE_C;
        }

        return fileName.substring(0, 13) + nextType + fileName.substring(14);
    }

    /**
     * カラー画像かどうか
     *
     * @param fileName ファイル名
     * @return boolean true - カラー、false - 白黒
     */
    private boolean isColor(String fileName) {
        if (StringUtil.isBlank(fileName)) {
            return false;
        }

        String name = null;
        int index = fileName.lastIndexOf(".");
        if (index < 0) {
            name = fileName;
        } else {
            name = fileName.substring(0, index);
        }

        if (name.length() < 14) {
            return false;
        }

        return !IMG_TYPE_WB.equals(name.substring(13, 14));
    }

    /**
     * 指定されたパスにあるファイルを取得する。
     * 所得したファイルは原文オブジェクトにセットされる。
     *
     * @param article 記事データ
     * @param resultList 結果リスト
     * @throws ServiceException エラー発生時
     */
    private void setFiles(ArticleVO article, ArrayList<String> resultList) throws ServiceException {
        OrgTextVO[] orgTextVoArray = article.getOrgTexts();
        File orgText = null;

        int index = 0;
        boolean[] orgnFlgs = createArray(orgTextVoArray);

        for (OrgTextVO orgTextVo : orgTextVoArray) {

            boolean exist = false;

            // パスがブランク
            if (StringUtil.isBlank(orgTextVo.getOrgTextPath())) {
                File emptyFile = new File(EMPTY_PDF_FILE_PATH);
                orgTextVo.setOrgText(emptyFile);
                orgTextVo.setOrgTextFileName(emptyFile.getName());
                resultList.add(FILE_NOT_EXIST);
                orgnFlgs[index] = false;
                // throw new ServiceException("原文のパスが指定されていません。");
            } else {
                String path = this.editPath(orgTextVo.getOrgTextPath());
                Logger.debug("OrgTextVO.getOrgTextPathに設定されたファイル名::" + path);
                Logger.info("PDFファイル [" + path + "]");

                orgText = new File(path);
                if (orgText.exists()) {
                    // ファイルあり
                    exist = true;
                } else {
                    // ファイルがなかった場合、JPEG→PNG→カラー→白黒の順でPDFの取得を試みる
                    String fileName = orgText.getName();
                    while (!exist && this.isColor(fileName)) {
                        fileName = this.convertFileName(fileName);
                        orgText =  new File(orgText.getParent(), fileName);
                        exist = orgText.exists();
                    }

                }

                orgTextVo.setOrgTextFileName(orgText.getName());
                if (exist) {
                    orgTextVo.setOrgText(orgText);
                    resultList.add(FILE_EXIST);
                    orgnFlgs[index] = true;
                } else {
                    orgTextVo.setOrgText(new File(EMPTY_PDF_FILE_PATH));
                    resultList.add(FILE_NOT_EXIST);
                    orgnFlgs[index] = false;
                }
            }

            index++;
        }

        article.setOrgFlgs(orgnFlgs);
    }

    /**
     * 空の配列を生成します
     *
     * @param args 原文配列
     * @return boolean[] 空の配列
     */
    private boolean[] createArray(OrgTextVO[] args) {
        if (args == null || args.length == 0) {
            return new boolean[0];
        }

        return new boolean[args.length];
    }

    /**
     * パスを編集する。<br>
     * プレフィックスが指定されていれば、<br>
     * パスにプレフィックスを追加して返却する。
     *
     * @param path パス
     * @return result 編集されたパス
     */
    private String editPath(String path) {
        try {
            return FileUtil.getFullPath(path, CifsType.PDF);
        } catch (CifsApplicationException e) {
            Logger.error("PDFファイルパスの取得に失敗しました：" + path, e);
            return path;
        }
    }

    /**
     * <p>FFSからTIFF画像を取得するためのファイルパスを生成する</p>
     * yyyy/yyyymm/yyyymmddxxxxx
     *
     * @param fileName ファイル名
     * @return Striing ファイルパス（相対）
     */
    private String editPathForFFS(String fileName) {
        if (StringUtil.isBlank(fileName) || fileName.length() < 6) {
            return "";
        }

        StringBuilder path = new StringBuilder();

        // トップディレクトリの設定があればそこから
        if (!StringUtil.isBlank(CIFS_OVER_YEAR_FILE_DIR)) {
            path.append(CIFS_OVER_YEAR_FILE_DIR);
            path.append(File.separator);
        }

        path.append(fileName.substring(0, 4));
        path.append(File.separator);
        path.append(fileName.substring(0, 6));
        path.append(File.separator);
        path.append(fileName);

        return path.toString().replaceAll(REPLACE_FILE_SEPARATOR, RETURN_FILE_SEPARATOR);
    }

    /**
     * 保存ファイル名を編集する。<br>
     * 引数に拡張子pdfを付加する。
     *
     * @param fileName ファイル名
     * @return result 編集されたファイル名
     */
    private String editSaveFileName(String fileName) {
        String result = fileName;

        if (!StringUtil.isBlank(result) && result.toLowerCase().indexOf("pdf") < 0) {
            result = result + ".pdf";
        }

        return result;
    }

    /**
     * 保存ディレクトリを生成し、そのパスを返す。<br>
     *
     * @param workDirName 作業ディレクトリ名
     * @return result されたファイル名
     */
    private String makeSaveDir(String workDirName) {
        StringBuffer result = new StringBuffer();

        String formatDir = getTodayDirName();

        if (!StringUtil.isBlank(FILE_SAVE_DIR)) {
            result.append(FILE_SAVE_DIR).append(File.separator).append(formatDir).append(File.separator);

            if (workDirName != null && !"".equals(workDirName)) {
                result.append(workDirName).append(File.separator);
            }
        }

        File saveDir = new File(result.toString());
        saveDir.mkdirs();

        return saveDir.getAbsolutePath();
    }

    /**
     * <p>FFSからのダウンロードファイル配置ディレクトリを返却します</p>
     * 【ルートディレクトリ】/【YYYYMMDD】/【作業ディレクトリ】
     *
     * @param workDirName 作業ディレクトリ
     * @return String FFSからのダウンロードファイル配置ディレクトリ
     */
    private String editSavePathForFFS(String workDirName) {
        StringBuffer result = new StringBuffer();

        String formatDir = getTodayDirName();

        if (!StringUtil.isBlank(CIFS_DL_DIR)) {
            result
                .append(CIFS_DL_DIR)
                .append(File.separator)
                .append(formatDir);

            if (workDirName != null && !"".equals(workDirName)) {
                result
                    .append(File.separator)
                    .append(workDirName);
            }
        }

        File saveDir = new File(result.toString());
        saveDir.mkdirs();

        return saveDir.getAbsolutePath();
    }

    /**
     * FFSからのダウンロードファイル配置ディレクトリ（YYYYMMDD）
     *
     * @return String FFSからのダウンロードファイル配置ディレクトリ（YYYYMMDD）
     */
    private String getTodayDirName() {
        String formatDir = null;

        Date today = new Date();
        SimpleDateFormat sdFormat = new SimpleDateFormat("yyyyMMdd");
        formatDir = sdFormat.format(today);

        return formatDir;
    }

    /**
     * 作業ディレクトリ名を返却します
     *
     * @param vo 記事VO
     * @return String 作業ディレクトリ名
     */
    private String getWorkDirName(ArticleOrgTextVO vo) {
        // 実行時間取得
        String date = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date());
        // UUID取得
        String uuid = UUID.randomUUID().toString();

        // VOがなかったら終了
        if (vo == null) {
            return date + DIR_NAME_SPLIT + uuid;
        }

        // ユーザID取得
        String userId = null;
        if (vo.getMcFlg()) {
            userId = vo.getDmsId();
        } else {
            userId = vo.getPrintUserId();
        }

        // ユーザID未指定なら終了
        if (userId == null || "".equals(userId)) {
            return date + DIR_NAME_SPLIT + uuid;
        }

        return date + DIR_NAME_SPLIT + userId + DIR_NAME_SPLIT + uuid;
    }

    /**
     * ファイルをコピーします
     *
     * @param binarySrc コピー元
     * @param destFile コピー先
     * @throws IOException エラー発生時
     */
    private void copy(byte[] binarySrc, File destFile) throws IOException {
        try (ByteArrayInputStream bais = new ByteArrayInputStream(binarySrc);
                FileOutputStream bos = new FileOutputStream(destFile)) {

            IOUtils.copy(bais, bos);
        }
    }
}